@@ -1,5 +1,5 @@
-#ifndef __INCLUDE__MY_SYSUTILS_H__
-#define __INCLUDE__MY_SYSUTILS_H__ 1
+#ifndef __INCLUDE_MY_UTILS_H__
+#define __INCLUDE_MY_UTILS_H__ 1
 
 #include <EXTERN.h>
 #include <perl.h>
@@ -8,14 +8,19 @@
 #define __PACKAGE__ "PAB3::Utils"
 
 #undef DWORD
-#define DWORD unsigned long
+#define DWORD unsigned int
+
+#undef HAS_UV64
+#if UVSIZE == 8
+#	define HAS_UV64 1
+#endif
 
 #undef XLONG
 #undef UXLONG
-#if defined __unix__
+#if defined __GNUC__ || defined __unix__ || defined __CYGWIN__ || defined __sun
 #	define XLONG long long
 #	define UXLONG unsigned long long
-#elif defined __WIN__
+#elif defined _WIN32
 #	define XLONG __int64
 #	define UXLONG unsigned __int64
 #else
@@ -60,25 +65,27 @@ typedef struct st_my_locale {
 	char		name[16];
 	char		decimal_point;
 	char		thousands_sep;
-	char		grouping;
+	char		grouping[4];
 	char		frac_digits;
 	char		int_frac_digits;
-	char		currency_symbol[4];
-	char		int_curr_symbol[4];
+	char		currency_symbol[7];
+	char		int_curr_symbol[7];
 	char		curr_symb_align;
-	char		int_curr_symb_align;
+	char		curr_symb_space;
 	char		negative_sign;
 	char		positive_sign;
-	char		short_date_format[16];
-	char		long_date_format[16];
-	char		short_time_format[16];
-	char		long_time_format[16];
-	char		time_am_string[8];
-	char		time_pm_string[8];
-	char		short_month_names[12][8];
-	char		long_month_names[12][16];
-	char		short_day_names[7][8];
-	char		long_day_names[7][16];
+	char		date_format[16];
+	char		time_format[16];
+	char		datetime_format[32];
+	char		ampm_format[16];
+	char		time_am_upper[16];
+	char		time_pm_upper[16];
+	char		time_am_lower[16];
+	char		time_pm_lower[16];
+	char		short_month_names[12][7];
+	char		long_month_names[12][32];
+	char		short_day_names[7][7];
+	char		long_day_names[7][32];
 } my_locale_t;
 
 typedef struct st_my_locale_alias {
@@ -89,25 +96,71 @@ typedef struct st_my_locale_alias {
 
 typedef struct st_my_thread_var {
 	struct st_my_thread_var		*prev, *next;
-	unsigned long				tid;
+	UV							tid;
 	my_locale_t					locale;
 	my_vtimezone_t				timezone;
 	my_vdatetime_t				time_struct;
 } my_thread_var_t;
 
-#define MY_CXT_KEY __PACKAGE__ "::_guts" XS_VERSION
+//#define MY_CXT_KEY __PACKAGE__ "::_guts" XS_VERSION
+
+#ifndef MAX_PATH
+#define MAX_PATH 512
+#endif
+
+#undef START_MY_CXT
+#undef MY_CXT_INIT
+#undef MY_CXT
+#undef dMY_CXT
+#undef pMY_CXT
+#undef aMY_CXT
+#undef MY_CXT_CLONE
+
+#define START_MY_CXT my_cxt_t my_cxtp;
+#define EXPORT_MY_CXT extern my_cxt_t my_cxtp;
+#define MY_CXT my_cxtp
+#define dMY_CXT dNOOP
+#define pMY_CXT void
+#define aMY_CXT
+#define MY_CXT_INIT \
+	Zero( &MY_CXT, 1, my_cxt_t )
+#define MY_CXT_CLONE
 
 typedef struct st_my_cxt {
-	char						locale_path[256]; 
-	char						zoneinfo_path[256]; 
+	char						state;
+	char						locale_path[MAX_PATH]; 
+	char						zoneinfo_path[MAX_PATH]; 
 	int							locale_path_length;
 	int							zoneinfo_path_length;
 	my_thread_var_t				*threads;
 	my_thread_var_t				*last_thread;
 	my_locale_alias_t			*locale_alias;
+	int							locale_alias_count;
+#ifdef USE_ITHREADS
+	perl_mutex					thread_lock;
+	PerlInterpreter				*perl;
+#endif
 } my_cxt_t;
 
-START_MY_CXT
+EXPORT_MY_CXT
+
+#ifdef USE_ITHREADS
+#define MY_CXT_LOCK \
+	MUTEX_LOCK( &MY_CXT.thread_lock )
+#define MY_CXT_UNLOCK \
+	MUTEX_UNLOCK( &MY_CXT.thread_lock )
+#else
+#define MY_CXT_LOCK
+#define MY_CXT_UNLOCK
+#endif
+
+#define FASTSTRCPY(dst,src) \
+	do { \
+		register const char *__src = (src); \
+		register char *__dst = (dst); \
+		for( ; *__src != '\0'; *__dst ++ = *__src ++ ); \
+		(dst) = __dst; \
+	} while( 0 )
 
 #define ISWHITECHAR(ch) \
 	( (ch) == 32 || (ch) == 10 || (ch) == 13 || (ch) == 9 || (ch) == 0 \
@@ -125,42 +178,32 @@ START_MY_CXT
 
 #define ARRAY_LEN(x) ( sizeof( (x) ) / sizeof( (x)[0] ) )
 
-static const double ROUND_PREC[] = {
-	1, 10, 1e2, 1e3, 1e4, 1e5, 1e6, 1e7, 1e8, 1e9, 1e10, 1e11, 1e12
-	, 1e13, 1e14, 1e15, 1e16, 1e17, 1e18, 1e19
-};
-static const int ROUND_PREC_MAX = 1 + (int) ARRAY_LEN( ROUND_PREC );
+extern const double ROUND_PREC[];
+extern const int ROUND_PREC_MAX;
 
-static const my_locale_t DEFAULT_LOCALE = {
-	"en_EN", '.', ',', 3, 2, 2, "$", "USD", 'l', 'l', '-', '+',
-	"%m/%d/%Y", "%a %b %d %Y", "%H:%M", "%H:%M:%S %Z", "AM", "PM",
-	{
-		"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep",
-		"Oct", "Nov", "Dec"
-	},
-	{
-		"January", "February", "March", "April", "May", "June", "July",
-		"August", "September", "October", "November", "December"
-	},
-	{ "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" },
-	{
-		"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday",
-		"Friday", "Saturday"
-	},
-};
-
-static const char *DEFAULT_ZONE = "GMT";
+extern const my_locale_t DEFAULT_LOCALE;
+extern const char *DEFAULT_ZONE;
 
 char *PerlIO_fgets( char *buf, size_t max, PerlIO *stream );
 
-char *my_strncpy( char *dst, const char *src, unsigned long len );
+char *my_strncpy( char *dst, const char *src, size_t len );
 char *my_strcpy( char *dst, const char *src );
+int my_stricmp( const char *cs, const char *ct );
 char *my_itoa( char* str, long value, int radix );
 char *my_ltoa( char* str, XLONG value, int radix );
 
+DWORD get_current_thread_id();
+
 #define find_or_create_tv(cxt,tv,tid) \
-	if( ! ( (tv) = find_thread_var( (cxt), (tid) ) ) ) \
-		(tv) = create_thread_var( (cxt), (tid) )
+	if( (cxt)->state > 0 ) { \
+		MY_CXT_LOCK; \
+		if( ! ((tv) = find_thread_var( (cxt), (tid) )) ) \
+			(tv) = create_thread_var( (cxt), (tid) ); \
+		MY_CXT_UNLOCK; \
+	} \
+	else { \
+		(tv) = NULL; \
+	}
 
 my_thread_var_t *find_thread_var( my_cxt_t *cxt, UV tid );
 my_thread_var_t *create_thread_var( my_cxt_t *cxt, UV tid );
@@ -183,10 +226,13 @@ size_t _int_strfmon(
 int parse_timezone( my_cxt_t *cxt, const char *tz, my_vtimezone_t *vtz );
 #define read_timezone parse_timezone
 my_vdatetime_t *apply_timezone( my_thread_var_t *tv, time_t *timer );
+char *_int_number_format2( double val, char *str, int fd, char dp,
+	char ts, const char *grp, char ns, char ps, int zf, char fc
+);
 char *_int_number_format(
 	double value, char *str, int maxlen, int fd, char dp, char ts, char ns,
 	char ps, int zf, char fc
 );
-double _my_round( double num, int prec );
+double my_round( double num, int prec );
 
 #endif
